PROGRAM Generate_Pkz3PT
  USE cosmo
  USE growth
  USE nonlinearpk
  ! A program for generating 2-dimensional arrays of non-linear 
  ! power spectra, P(k,z), as a function of wavenumbers k and redshifts z. 
  ! It uses the 3rd-order perturbation theory (3PT). 
  ! - k is in units of h Mpc^-1
  ! - P(k) is in units of h^-3 Mpc^3
  ! This code will generate large files containing P(k,z)  as a 
  ! function of both k AND z.
  ! January 2, 2012: E.Komatsu
  IMPLICIT none
  integer :: nk=691 ! # of data points in wavenumber
  integer :: nz=901 ! # of data points in redshift
  character(len=128) :: filename
  integer :: ik,iz,n
  double precision :: g,z,dz,D
  double precision :: nonlinear_pkd,k_ov_h,dlnk,lnk,zin
  external g,nonlinear_pkd
! Specify three cosmological parameters
! The data type has been defined in MODULE cosmo.
  ode0=0.723d0
  om0=0.277d0
  w=-1d0
! tabulate g(z) by calling "setup_growth"
  call setup_growth
! read in and tabulate P(k)
  filename='wavenumber_pkd11_pkd22_pkd13_at_z=30.txt'
  n=691    ! # of lines in the file
  zin=30d0 ! redshift of the input power spectrum
! output P(k,z)
  open(11,file='redshift_wavenumber_pk3pt.txt',status='unknown')
! Loop over redshifts
  z=0d0
  dz=1d-2
  do iz=1,nz
     D=g(z)/g(zin)*(1d0+zin)/(1d0+z) ! linear growth relative to zin
     CALL open_nonlinearpkd(filename,n,D)
     ! Loop over wavenumbers 
     k_ov_h=1d-4 ! h/Mpc
     dlnk=2d-2
     do ik=1,nk
        write(11,'(3E18.8)')z,k_ov_h,nonlinear_pkd(k_ov_h)
        lnk=dlog(k_ov_h)+dlnk
        k_ov_h=dexp(lnk)
     enddo
     z=z+dz
     call close_nonlinearpkd
  enddo
  print*,'the number of data points in wavenumber=',nk,'from k=1e-4 to',dexp(lnk-dlnk),'h/Mpc'
  print*,'the number of data points in redshift=',nz,'from z=0 to',z-dz
  close(11)
END PROGRAM Generate_Pkz3PT
